/*
 * Copyright (c) 2022, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 *
 */

#ifndef SHARE_GC_G1_G1CONCURRENTREFINETHREADSNEEDED_HPP
#define SHARE_GC_G1_G1CONCURRENTREFINETHREADSNEEDED_HPP

#include "memory/allocation.hpp"
#include "utilities/globalDefinitions.hpp"

class G1Analytics;
class G1Policy;

// Used to compute the number of refinement threads that need to be running
// in order to have the number of pending cards below the policy-directed
// goal when the next GC occurs.
class G1ConcurrentRefineThreadsNeeded : public CHeapObj<mtGC> {
  G1Policy* _policy;
  double _update_period_ms;
  double _predicted_time_until_next_gc_ms;
  size_t _predicted_cards_at_next_gc;
  uint _threads_needed;

public:
  G1ConcurrentRefineThreadsNeeded(G1Policy* policy, double update_period_ms);

  // Update the number of running refinement threads needed to reach the
  // target before the next GC.
  void update(uint active_threads,
              size_t available_bytes,
              size_t num_cards,
              size_t target_num_cards);

  // Estimate of the number of active refinement threads needed to reach the
  // target before the next GC.
  uint threads_needed() const { return _threads_needed; }

  // Estimate of the time until the next GC.
  double predicted_time_until_next_gc_ms() const {
    return _predicted_time_until_next_gc_ms;
  }

  // Estimate of the number of pending cards at the next GC if no further
  // refinement is performed.
  size_t predicted_cards_at_next_gc() const {
    return _predicted_cards_at_next_gc;
  }
};

#endif // SHARE_GC_G1_G1CONCURRENTREFINETHREADSNEEDED_HPP
